PROGRAM PadOut;

{A Turbo Pascal (tm) program to pad out files, such as WordPerfect (tm) files,
with spaces (Hex 20) to the nearest 1024 bytes or 128 bytes, for 'clean'
transfer by Ymodem (1k Xmodem) or Xmodem, respectively. I would like to
thank Michael Reuben, who pointed out the problem of Xmodem padding,
particularly with WordPerfect (tm) files, to me, and motivated me to write
this program.

Program written by Bill Larkins, July 4, 1990. Hereby dedicated to the
public domain.}

USES
    Crt;
VAR
     FileName  : String[64];
     Switches  : String[4];
     InFile,
     OutFile   : String[64];
     ValidName  : Integer;
     StartFile : String[64];
     DiffName  : Boolean;
     XPad : Integer;
     XMod : Boolean;
     DotIn, SlashIn : Integer;
     DotOut, SlashOut : Integer;
     I      : Integer;
     GoodName : Integer;
     XIn, XOut : Integer;
     Sigmaa : Integer;
     Sigmab : Integer;

PROCEDURE CommandLine;

{Gets Command Line parameters}

BEGIN
     ValidName := 1;
     FileName := ParamStr(1);
     IF Length( FileName ) > 0 THEN
     InFile := FileName ELSE ValidName := 0;
     FileName := ParamStr(2);
     IF Length( FileName ) > 0 THEN
     OutFile := FileName ELSE ValidName := 0;
     Switches := ParamStr(3);
     IF ( ValidName < 1 ) THEN Writeln ( 'Invalid Filenames! Program Halted!');
     IF ( ValidName < 1 ) THEN HALT ELSE
END; {CommandLine}



PROCEDURE GetFiles;

BEGIN
   Write( 'Input File? ' ); Readln( FileName );
   StartFile := FileName;
   InFile := FileName;
   REPEAT
       Writeln( 'Output Filename MUST BE DIFFERENT Than Input Filename' );
       Write( 'Output File?' );
       Readln( FileName );
       OutFile := FileName ;
       DiffName := (FileName <> StartFile )
    UNTIL DiffName;

    Write( 'Type 128 to pad to 128 Bytes, the default is 1024 padding  ');
    Readln( Switches );

END; {GetFiles }


PROCEDURE CheckNames;

{Needed Variables are InFile, OutFile, and Switches}

BEGIN

    XMod := ( Switches <> '128' );
    IF Xmod THEN XPad := 0
            ELSE XPad := 1;

    DotIn := Pos( '.' , InFile );
    DotOut := Pos( '.' , OutFile );
    SlashIn := Pos( '\', InFile );
    SlashOut := Pos( '\', OutFile );
    GoodName := 0;
    XIn := Length( InFile) - DotIn; { XIn = length of extension if InFile}
    XOut := Length( OutFile) - DotOut; {XOut = Length of extension of
                Outfile}
    IF ( XIn > 3) THEN XIn := 3;
    IF ( XOut >3) THEN XOut := 3;
    IF ( XIn <> XOut) THEN GoodName := 1 ELSE
    Sigmaa := DotIn;
    Sigmab := DotIn + XIn;
    FOR I := Sigmaa to Sigmab DO
     IF ( InFile[I] <> OutFile[I] ) THEN GoodName := GoodName + 1;

    XIn := SlashIn - DotIn;
    XOut := SlashOut - DotOut;  {Now XIn, XOut are first part of filenames}
    IF ( XIn > 8) THEN XIn := 8;
    IF ( XOut > 8) THEN XOut := 8;
    IF ( XIn <> XOut ) THEN GoodName := GoodName + 1;
    Sigmaa := SlashIn +1;
    Sigmab := SlashIn + XIn;
    FOR I := Sigmaa to Sigmab DO
        IF ( InFile[I] <> OutFile[I] ) THEN GoodName := GoodName + 1;
    IF ( GoodName < 1 ) THEN Write( 'Invalid FileNames, Program Halted!' );
    IF (GoodName > 0) THEN Writeln ( 'Valid Filenames-Proceeding') ELSE HALT
END; {CheckNames}

PROCEDURE ProcessX;

VAR
    InF, OutF : FILE OF Byte;
    OneByte : Byte;
    Size : Real;
    CountDown : Real;
    Padding : Real;
    Ch : Byte;

BEGIN
     Assign ( InF, InFile );
     Reset( InF );
     Assign( OutF, OutFile );
     Rewrite( OutF );
     Size := 0.0;
     CountDown :=0.0;
     Padding :=0.0;
     Ch := 32;
     WHILE Size < FileSize( InF ) DO
     BEGIN
          Read( InF, OneByte );
          Write( OutF, OneByte );
          Size := Size + 1
     END;
     CountDown := Size;
     REPEAT
           CountDown := CountDown -128
     UNTIL CountDown < 0;
     REPEAT
           Write( OutF, Ch );
           CountDown := CountDown +1;
           Padding := Padding +1
     UNTIL CountDown > -1;
     Writeln( Size:1:0, ' bytes copied' );
     Writeln (Padding:1:0, ' padding spaces added');
     Close ( InF );
     Close ( OutF );
END; {ProcessX}

PROCEDURE ProcessY;

VAR
    InF, OutF : FILE OF Byte;
    OneByte : Byte;
    Size : Real;
    CountDown : Real;
    Padding : Real;
    Ch : Byte;

BEGIN
     Assign ( InF, InFile );
     Reset( InF );
     Assign( OutF, OutFile );
     Rewrite( OutF );
     Size := 0.0;
     CountDown :=0.0;
     Padding :=0.0;
     Ch := 32;
     WHILE Size < FileSize( InF ) DO
     BEGIN
          Read( InF, OneByte );
          Write( OutF, OneByte );
          Size := Size + 1
     END;
     CountDown := Size;
     REPEAT
           CountDown := CountDown -1024
     UNTIL CountDown < 0;
     REPEAT
           Write( OutF, Ch );
           CountDown := CountDown +1;
           Padding := Padding +1
     UNTIL CountDown > -1;
     Writeln( Size:1:0, ' bytes copied' );
     Writeln (Padding:1:0, ' padding spaces added');
     Close (InF );
     Close (OutF );

END; {ProcessY}



BEGIN
   ClrScr;
   Writeln( 'Pad a file with Spaces to the nearest 1024 or 128 bytes' );
   Writeln( '          for clean 1k Xmodem or 128 Xmodem Transfer' );
   Writeln;
   Writeln( '     Program written by Bill Larkins, sysop,' );
   Writeln( ' Chevy Chase Board BBS (703)549-5574, July 1990' );
   Writeln( '      Program released to the Public Domain' );
   Writeln;
   Writeln( 'Disclaimer: this program is distributed As Is, and has not' );
   Writeln( 'been tested on a range of machines. Use at your own risk' );
   Writeln( 'You MUST use a valid Output Filename different than the Input ');
   Writeln(  ' Filename!' );
     Writeln;

  IF ParamCount <= 1
        THEN GetFiles
        ELSE CommandLine;

    CheckNames;

    IF Xpad = 1 THEN ProcessX
                ELSE ProcessY;

END.
